@@ -29,7 +29,8 @@ from matplotlib.artist import Artist
 from matplotlib.figure import Figure
 from matplotlib.text import Text
 from matplotlib.axes import Axes
-from qtpy import QtCore, QtWidgets, QtGui
+from matplotlib.backends.qt_compat import QtCore, QtGui, QtWidgets
+from .helper_functions import main_figure
 
 
 class Linkable:
@@ -39,6 +40,7 @@ class Linkable:
     def link(self, property_name: str, signal: QtCore.Signal = None, condition: callable = None, direct: bool = False):
         self.element = None
         self.direct = direct
+        self.property_name = property_name
         if direct:
             parts = property_name.split(".")
             s = self
@@ -61,11 +63,16 @@ class Linkable:
             self.getLinkedProperty = get
             self.serializeLinkedProperty = lambda x: "." + property_name + " = %s" % x
         else:
-            def set(v):
+            def set(v, v_list=None):
+                if v_list is None:
+                    v = [v]+[v]*len(main_figure(self.element).selection.targets)
+                else:
+                    v = v_list
+
                 # special treatment for the xylabels, as they are not directly the target objects
                 label_object = None
-                if isinstance(self.element, Text) and isinstance(self.element.figure.selection.targets[0].target, Axes):
-                    for elm in self.element.figure.selection.targets:
+                if isinstance(self.element, Text) and len(main_figure(self.element).selection.targets) and isinstance(main_figure(self.element).selection.targets[0].target, Axes):
+                    for elm in main_figure(self.element).selection.targets:
                         elm = elm.target
                         if self.element == getattr(getattr(elm, f"get_xaxis")(), "get_label")():
                             label_object = "x"
@@ -75,24 +82,52 @@ class Linkable:
                             break
 
                 elements = []
-                getattr(self.element, "set_" + property_name)(v)
+                getattr(self.element, "set_" + property_name)(v[0])
                 elements.append(self.element)
-                for elm in self.element.figure.selection.targets:
+                index = 0
+                for elm in main_figure(self.element).selection.targets:
                     elm = elm.target
                     # special treatment for the xylabels, as they are not directly the target objects
                     if label_object is not None:
                         elm = getattr(getattr(elm, f"get_{label_object}axis")(), "get_label")()
                     if elm != self.element:
                         try:
-                            getattr(elm, "set_" + property_name, None)(v)
+                            index += 1
+                            getattr(elm, "set_" + property_name, None)(v[index])
                         except TypeError as err:
                             pass
                         else:
                             elements.append(elm)
                 return elements
 
+            def getAll():
+                label_object = None
+                if isinstance(self.element, Text) and len(main_figure(self.element).selection.targets) and isinstance(main_figure(self.element).selection.targets[0].target, Axes):
+                    for elm in main_figure(self.element).selection.targets:
+                        elm = elm.target
+                        if self.element == getattr(getattr(elm, f"get_xaxis")(), "get_label")():
+                            label_object = "x"
+                            break
+                        if self.element == getattr(getattr(elm, f"get_yaxis")(), "get_label")():
+                            label_object = "y"
+                            break
+
+                values = [(self.element, property_name, getattr(self.element, "get_" + property_name)())]
+                for index, elm in enumerate(main_figure(self.element).selection.targets):
+                    elm = elm.target
+                    # special treatment for the xylabels, as they are not directly the target objects
+                    if label_object is not None:
+                        elm = getattr(getattr(elm, f"get_{label_object}axis")(), "get_label")()
+                    if elm != self.element:
+                        try:
+                            values.append([elm, property_name, getattr(elm, "get_" + property_name, None)()])
+                        except TypeError as err:
+                            pass
+                return values
+
             self.setLinkedProperty = set  # lambda text: getattr(self.element, "set_"+property_name)(text)
             self.getLinkedProperty = lambda: getattr(self.element, "get_" + property_name)()
+            self.getLinkedPropertyAll = getAll
             self.serializeLinkedProperty = lambda x: ".set_" + property_name + "(%s)" % x
 
         if condition is None:
@@ -116,17 +151,44 @@ class Linkable:
 
     def updateLink(self):
         """ update the linked property """
+        old_value = self.getLinkedPropertyAll()
+
         try:
             elements = self.setLinkedProperty(self.get())
         except AttributeError:
             return
-        for element in elements:
+
+        new_value = self.getLinkedPropertyAll()
+
+        def save_change(element):
             if isinstance(element, mpl.figure.Figure):
                 fig = element
             else:
-                fig = element.figure
-            fig.change_tracker.addChange(element, self.serializeLinkedProperty(self.getSerialized()))
+                fig = main_figure(element)
+
+            if isinstance(element, Text):
+                fig.change_tracker.addNewTextChange(element)
+            else:
+                fig.change_tracker.addChange(element, self.serializeLinkedProperty(self.getSerialized()))
+
+        def undo():
+            for elem, property_name, value in old_value:
+                getattr(elem, "set_" + property_name, None)(value)
+                save_change(elem)
+        def redo():
+            for elem, property_name, value in new_value:
+                getattr(elem, "set_" + property_name, None)(value)
+                save_change(elem)
+
+        element = elements[0]
+        if isinstance(element, mpl.figure.Figure):
+            fig = element
+        else:
+            fig = main_figure(element)
+
+        fig.change_tracker.addEdit([undo, redo, "Change property"])
         fig.canvas.draw()
+        main_figure(self.element).signals.figure_selection_property_changed.emit()
 
     def set(self, value):
         """ set the value (to be overloaded) """
@@ -155,6 +217,7 @@ class FreeNumberInput(QtWidgets.QLineEdit):
             valueChanged : a signal that is emitted when the value is changed by the user
         """
         QtWidgets.QLineEdit.__init__(self)
+        #self.setMaximumWidth(50)
         self.textChanged.connect(self.emitValueChanged)
 
     def emitValueChanged(self):
@@ -190,6 +253,8 @@ class FreeNumberInput(QtWidgets.QLineEdit):
 
 class DimensionsWidget(QtWidgets.QWidget, Linkable):
     valueChanged = QtCore.Signal(tuple)
+    valueChangedX = QtCore.Signal(float)
+    valueChangedY = QtCore.Signal(float)
     transform = None
     noSignal = False
 
@@ -218,7 +283,8 @@ class DimensionsWidget(QtWidgets.QWidget, Linkable):
             self.input1.setSingleStep(0.1)
             self.input1.setMaximum(99999)
             self.input1.setMinimum(-99999)
-        self.input1.valueChanged.connect(self.onValueChanged)
+            self.input1.setMaximumWidth(100)
+        self.input1.valueChanged.connect(self.onValueChangedX)
         self.layout.addWidget(self.input1)
 
         self.text2 = QtWidgets.QLabel(join)
@@ -233,7 +299,8 @@ class DimensionsWidget(QtWidgets.QWidget, Linkable):
             self.input2.setSingleStep(0.1)
             self.input2.setMaximum(99999)
             self.input2.setMinimum(-99999)
-        self.input2.valueChanged.connect(self.onValueChanged)
+            self.input2.setMaximumWidth(100)
+        self.input2.valueChanged.connect(self.onValueChangedY)
         self.layout.addWidget(self.input2)
 
         self.editingFinished = self.valueChanged
@@ -251,12 +318,24 @@ class DimensionsWidget(QtWidgets.QWidget, Linkable):
         """ set the transform for the units """
         self.transform = transform
 
+    def onValueChangedX(self):
+        """ called when the value was changed -> emit the value changed signal """
+        if not self.noSignal:
+            self.valueChangedX.emit(self.value()[0])
+            self.valueChanged.emit(tuple(self.value()))
+
+    def onValueChangedY(self):
+        """ called when the value was changed -> emit the value changed signal """
+        if not self.noSignal:
+            self.valueChangedY.emit(self.value()[1])
+            self.valueChanged.emit(tuple(self.value()))
+
     def onValueChanged(self):
         """ called when the value was changed -> emit the value changed signal """
         if not self.noSignal:
             self.valueChanged.emit(tuple(self.value()))
 
-    def setValue(self, values: tuple):
+    def setValue(self, values: tuple, signal=False):
         """ set the two values """
         self.noSignal = True
         if self.transform:
@@ -264,6 +343,8 @@ class DimensionsWidget(QtWidgets.QWidget, Linkable):
         self.input1.setValue(values[0])
         self.input2.setValue(values[1])
         self.noSignal = False
+        if signal is True:
+            self.onValueChanged()
 
     def value(self):
         """ get the value """
@@ -290,7 +371,7 @@ class TextWidget(QtWidgets.QWidget, Linkable):
     noSignal = False
     last_text = None
 
-    def __init__(self, layout: QtWidgets.QLayout, text: str, multiline: bool = False, horizontal: bool = True):
+    def __init__(self, layout: QtWidgets.QLayout, text: str, multiline: bool = False, horizontal: bool = True, allow_literal_decoding=False):
         """ a text input widget with a label.
 
         Args:
@@ -301,6 +382,7 @@ class TextWidget(QtWidgets.QWidget, Linkable):
         """
         QtWidgets.QWidget.__init__(self)
         layout.addWidget(self)
+        self.allow_literal_decoding = allow_literal_decoding
         if horizontal:
             self.layout = QtWidgets.QHBoxLayout(self)
         else:
@@ -328,7 +410,7 @@ class TextWidget(QtWidgets.QWidget, Linkable):
         """ set the text of the label """
         self.label.setLabel(text)
 
-    def setText(self, text: str):
+    def setText(self, text: str, signal=False):
         """ set contents of the text input widget """
         self.noSignal = True
         text = text.replace("\n", "\\n")
@@ -338,6 +420,8 @@ class TextWidget(QtWidgets.QWidget, Linkable):
         else:
             self.input1.setText(text)
         self.noSignal = False
+        if signal is True:
+            self.editingFinished.emit()
 
     def text(self) -> str:
         """ return the text """
@@ -345,18 +429,23 @@ class TextWidget(QtWidgets.QWidget, Linkable):
         return text.replace("\\n", "\n")
 
     def get(self) -> str:
+        import ast
         """ get the value (used for the Linkable parent class) """
+        if self.allow_literal_decoding:
+            try:
+                return ast.literal_eval(self.text())
+            except ValueError:
+                return self.text()
         return self.text()
 
     def set(self, value: str):
         """ set the value (used for the Linkable parent class) """
-        self.setText(value)
+        self.setText(str(value))
 
     def getSerialized(self) -> str:
         """ serialize the value (used for the Linkable parent class) """
         return "\"" + str(self.get()) + "\""
 
-
 class NumberWidget(QtWidgets.QWidget, Linkable):
     editingFinished = QtCore.Signal()
     noSignal = False
@@ -396,11 +485,13 @@ class NumberWidget(QtWidgets.QWidget, Linkable):
         """ set the text label """
         self.label.setLabel(text)
 
-    def setValue(self, text: float):
+    def setValue(self, text: float, signal=False):
         """ set the value of the spin box """
         self.noSignal = True
         self.input1.setValue(text)
         self.noSignal = False
+        if signal is True:
+            self.editingFinished.emit()
 
     def value(self) -> float:
         """ get the value of the spin box """
@@ -457,12 +548,14 @@ class ComboWidget(QtWidgets.QWidget, Linkable):
         """ set the text of the label """
         self.label.setLabel(text)
 
-    def setText(self, text: str):
+    def setText(self, text: str, signal=False):
         """ set the value of the combo box """
         self.noSignal = True
         index = self.values.index(text)
         self.input1.setCurrentIndex(index)
         self.noSignal = False
+        if signal is True:
+            self.editingFinished.emit()
 
     def text(self) -> str:
         """ get the value of the combo box """
@@ -512,11 +605,14 @@ class CheckWidget(QtWidgets.QWidget, Linkable):
             self.stateChanged.emit(self.input1.isChecked())
             self.editingFinished.emit()
 
-    def setChecked(self, state: bool):
+    def setChecked(self, state: bool, signal=False):
         """ set the value of the check box """
         self.noSignal = True
         self.input1.setChecked(state)
         self.noSignal = False
+        if signal:
+            self.stateChanged.emit(self.input1.isChecked())
+            self.editingFinished.emit()
 
     def isChecked(self) -> bool:
         """ get the value of the checkbox """
@@ -625,7 +721,7 @@ class QColorWidget(QtWidgets.QWidget, Linkable):
     def OpenDialog(self):
         """ open a color chooser dialog """
         # get new color from color picker
-        self.current_color = QtGui.QColor(*tuple(mpl.colors.to_rgba_array(self.getColor())[0] * 255))
+        self.current_color = QtGui.QColor(*tuple(int(x) for x in mpl.colors.to_rgba_array(self.getColor())[0] * 255))
         self.dialog = QtWidgets.QColorDialog(self.current_color, self.parent())
         self.dialog.setOptions(QtWidgets.QColorDialog.ShowAlphaChannel)
         for index, color in enumerate(plt.rcParams['axes.prop_cycle'].by_key()['color']):
