@@ -19,24 +19,29 @@
 # You should have received a copy of the GNU General Public License
 # along with Pylustrator. If not, see <http://www.gnu.org/licenses/>
 
-from __future__ import division, print_function
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.artist import Artist
 from matplotlib.figure import Figure
+from matplotlib.axes import Axes
 from matplotlib.text import Text
 from matplotlib.patches import Rectangle, Ellipse
 from matplotlib.backend_bases import MouseEvent, KeyEvent
 from typing import Sequence
+from matplotlib.backends.qt_compat import QtCore, QtGui, QtWidgets
 
 from .snap import TargetWrapper, getSnaps, checkSnaps, checkSnapsActive, SnapBase
 from .change_tracker import ChangeTracker
+from pylustrator.change_tracker import UndoRedo
+import time
 
 DIR_X0 = 1
 DIR_Y0 = 2
 DIR_X1 = 4
 DIR_Y1 = 8
 
+blit = False
+
 
 class GrabFunctions(object):
     """ basic functionality used by all grabbers """
@@ -86,6 +91,16 @@ class GrabFunctions(object):
 
         self.snaps = getSnaps(self.targets, self.dir, no_height=self.no_height)
 
+        if blit is True:
+            for target in self.targets:
+                target.target.set_animated(True)
+
+            self.figure.canvas.draw()
+            self.bg = self.figure.canvas.copy_from_bbox(self.figure.bbox)
+        else:
+            pass
+        self.time = time.time()
+
     def releasedEvent(self, event: MouseEvent):
         """ when the mouse is released """
         for snap in self.snaps:
@@ -94,6 +109,12 @@ class GrabFunctions(object):
 
         self.parent.end_move()
 
+        if blit is True:
+            for target in self.targets:
+                target.target.set_animated(False)
+        else:
+            pass
+
     def movedEvent(self, event: MouseEvent):
         """ when the mouse is moved """
         if len(self.targets) == 0:
@@ -107,21 +128,37 @@ class GrabFunctions(object):
 
         self.parent.move([dx, dy], self.dir, self.snaps, keep_aspect_ratio=keep_aspect, ignore_snaps=ignore_snaps)
 
+        if blit is True:
+            fig = self.figure
+            fig.canvas.restore_region(self.bg)
+            for target in self.targets:
+                fig.draw_artist(target.target)
+            # copy the image to the GUI state, but screen might not be changed yet
+            fig.canvas.blit(fig.bbox)
+            # flush any pending GUI events, re-painting the screen if needed
+            fig.canvas.flush_events()
+        else:
+            self.figure.canvas.schedule_draw()
 
 class GrabbableRectangleSelection(GrabFunctions):
     grabbers = None
 
     def addGrabber(self, x: float, y: float, dir: int, GrabberClass: object):
         # add a grabber object at the given coordinates
-        self.grabbers.append(GrabberClass(self, x, y, dir))
+        self.grabbers.append(GrabberClass(self, x, y, dir, self.graphics_scene))
 
-    def __init__(self, figure: Figure):
+    def __init__(self, figure: Figure, graphics_scene=None):
         self.grabbers = []
         pos = [0, 0, 0, 0]
         self.positions = np.array(pos, dtype=float)
         self.p1 = self.positions[:2]
         self.p2 = self.positions[2:]
         self.figure = figure
+        self.graphics_scene = graphics_scene
+        self.graphics_scene_myparent = QtWidgets.QGraphicsRectItem(0, 0, 0, 0, self.graphics_scene)
+        self.graphics_scene_snapparent = QtWidgets.QGraphicsRectItem(0, 0, 0, 0, self.graphics_scene)
+        figure._pyl_graphics_scene_snapparent = self.graphics_scene_snapparent
+
 
         GrabFunctions.__init__(self, self, DIR_X0 | DIR_X1 | DIR_Y0 | DIR_Y1, no_height=True)
 
@@ -151,16 +188,38 @@ class GrabbableRectangleSelection(GrabFunctions):
 
         self.targets.append(target)
 
-        x0, y0, x1, y1 = np.min(new_points[:, 0]), np.min(new_points[:, 1]), np.max(new_points[:, 0]), np.max(
-            new_points[:, 1])
-        rect1 = Rectangle((x0, y0), x1 - x0, y1 - y0, picker=False, figure=self.figure, linestyle="-", edgecolor="w",
-                          facecolor="#FFFFFF00", zorder=900, label="_rect for %s" % str(target))
-        rect2 = Rectangle((x0, y0), x1 - x0, y1 - y0, picker=False, figure=self.figure, linestyle="--", edgecolor="k",
-                          facecolor="#FFFFFF00", zorder=900, label="_rect2 for %s" % str(target))
-        self.figure.patches.append(rect1)
-        self.figure.patches.append(rect2)
-        self.targets_rects.append(rect1)
-        self.targets_rects.append(rect2)
+        if new_points.shape[0] == 3:
+            x0, y0, x1, y1 = np.min(new_points[1:, 0]), np.min(new_points[1:, 1]), np.max(
+                new_points[1:, 0]), np.max(
+                new_points[1:, 1])
+        else:
+            x0, y0, x1, y1 = np.min(new_points[:, 0]), np.min(new_points[:, 1]), np.max(new_points[:, 0]), np.max(
+                new_points[:, 1])
+        if 0:
+
+            rect1 = Rectangle((x0, y0), x1 - x0, y1 - y0, picker=False, figure=self.figure, linestyle="-", edgecolor="w",
+                              facecolor="#FFFFFF00", zorder=900, label="_rect for %s" % str(target))
+            rect2 = Rectangle((x0, y0), x1 - x0, y1 - y0, picker=False, figure=self.figure, linestyle="--", edgecolor="k",
+                              facecolor="#FFFFFF00", zorder=900, label="_rect2 for %s" % str(target))
+            self.figure.patches.append(rect1)
+            self.figure.patches.append(rect2)
+            self.targets_rects.append(rect1)
+            self.targets_rects.append(rect2)
+        else:
+            pen1 = QtGui.QPen(QtGui.QColor("white"), 2)
+            pen2 = QtGui.QPen(QtGui.QColor("black"), 2)
+            pen2.setStyle(QtCore.Qt.DashLine)
+            pen3 = QtGui.QPen(QtGui.QColor("black"), 2)
+            brush1 = QtGui.QBrush(QtGui.QColor("red"))
+
+            w0, h0 = x1 - x0, y1 - y0
+            rect1 = QtWidgets.QGraphicsRectItem(x0, y0, w0, h0, self.graphics_scene_myparent)
+            rect1.setPen(pen1)
+            rect2 = QtWidgets.QGraphicsRectItem(x0, y0, w0, h0, self.graphics_scene_myparent)
+            rect2.setPen(pen2)
+
+            self.targets_rects.append(rect1)
+            self.targets_rects.append(rect2)
 
         self.update_extent()
 
@@ -175,6 +234,9 @@ class GrabbableRectangleSelection(GrabFunctions):
             else:
                 points = np.concatenate((points, new_points))
 
+        if points is None:
+            return
+
         for grabber in self.grabbers:
             grabber.targets = self.targets
 
@@ -200,9 +262,12 @@ class GrabbableRectangleSelection(GrabFunctions):
 
         if mode == "group":
             from pylustrator.helper_functions import axes_to_grid
-            return axes_to_grid([target.target for target in self.targets], track_changes=True)
+            #return axes_to_grid([target.target for target in self.targets], track_changes=True)
+            with UndoRedo([target.target for target in self.targets if isinstance(target.target, Axes)], "Grid Align"):
+                axes_to_grid([target.target for target in self.targets if isinstance(target.target, Axes)], track_changes=False)
 
         def align(y: int, func: callable):
+            self.start_move()
             centers = []
             for target in self.targets:
                 new_points = np.array(target.get_positions())
@@ -213,8 +278,14 @@ class GrabbableRectangleSelection(GrabFunctions):
                 new_points[:, y] += new_center - centers[index]
                 target.set_positions(new_points)
             self.update_extent()
+            self.has_moved = True
+            self.end_move()
+
+            self.figure.canvas.draw()
+            self.update_selection_rectangles()
 
         def distribute(y: int):
+            self.start_move()
             sizes = []
             positions = []
             for target in self.targets:
@@ -231,6 +302,11 @@ class GrabbableRectangleSelection(GrabFunctions):
                 new_points[:, y] += pos - np.min(new_points[:, y])
                 target.set_positions(new_points)
                 pos += sizes[index] + spaces
+            self.has_moved = True
+            self.end_move()
+
+            self.figure.canvas.draw()
+            self.update_selection_rectangles()
 
         if mode == "center_x":
             align(0, np.mean)
@@ -256,19 +332,35 @@ class GrabbableRectangleSelection(GrabFunctions):
         if mode == "distribute_y":
             distribute(1)
 
-    def update_selection_rectangles(self):
+        self.figure.signals.figure_selection_moved.emit()
+
+    def update_selection_rectangles(self, use_previous_offset=False):
         """ update the selection visualisation """
         if len(self.targets) == 0:
             return
-        for index, target in enumerate(self.targets):
-            new_points = np.array(target.get_positions())
-            for i in range(2):
-                rect = self.targets_rects[index*2+i]
-                rect.set_xy(new_points[0])
-                rect.set_width(new_points[1][0] - new_points[0][0])
-                rect.set_height(new_points[1][1] - new_points[0][1])
-
-        self.update_extent()
+        if 0:
+            for index, target in enumerate(self.targets):
+                new_points = np.array(target.get_positions())
+                for i in range(2):
+                    rect = self.targets_rects[index*2+i]
+                    rect.set_xy(new_points[0])
+                    rect.set_width(new_points[1][0] - new_points[0][0])
+                    rect.set_height(new_points[1][1] - new_points[0][1])
+        else:
+            for index, target in enumerate(self.targets):
+                new_points = np.array(target.get_positions(use_previous_offset, update_offset=True))
+                if new_points.shape[0] == 3:
+                    x0, y0, x1, y1 = np.min(new_points[1:, 0]), np.min(new_points[1:, 1]), np.max(
+                        new_points[1:, 0]), np.max(
+                        new_points[1:, 1])
+                else:
+                    x0, y0, x1, y1 = np.min(new_points[:, 0]), np.min(new_points[:, 1]), np.max(
+                        new_points[:, 0]), np.max(
+                        new_points[:, 1])
+                w0, h0 = x1 - x0, y1 - y0
+                for i in range(2):
+                    rect = self.targets_rects[index * 2 + i]
+                    rect.setRect(x0, y0, w0, h0)
 
     def remove_target(self, target: Artist):
         """ remove an artist from the current selection """
@@ -279,8 +371,10 @@ class GrabbableRectangleSelection(GrabFunctions):
         self.targets.pop(index)
         rect1 = self.targets_rects.pop(index*2)
         rect2 = self.targets_rects.pop(index*2)
-        self.figure.patches.remove(rect1)
-        self.figure.patches.remove(rect2)
+        rect1.scene().removeItem(rect1)
+        rect2.scene().removeItem(rect2)
+        #self.figure.patches.remove(rect1)
+        #self.figure.patches.remove(rect2)
         if len(self.targets) == 0:
             self.clear_targets()
         else:
@@ -302,7 +396,8 @@ class GrabbableRectangleSelection(GrabFunctions):
     def clear_targets(self):
         """ remove all elements from the selection """
         for rect in self.targets_rects:
-            self.figure.patches.remove(rect)
+            self.graphics_scene.scene().removeItem(rect)
+            #self.figure.patches.remove(rect)
         self.targets_rects = []
         self.targets = []
 
@@ -370,16 +465,18 @@ class GrabbableRectangleSelection(GrabFunctions):
         self.start_p1 = self.p1.copy()
         self.start_p2 = self.p2.copy()
         self.hide_grabber()
+        self.has_moved = False
 
         self.store_start = self.get_save_point()
 
     def end_move(self):
         """ a grabber move stopped """
         self.update_grabber()
-        self.figure.canvas.draw()
 
         self.store_end = self.get_save_point()
-        self.figure.change_tracker.addEdit([self.store_start, self.store_end])
+        if self.has_moved is True:
+            self.figure.signals.figure_selection_moved.emit()
+            self.figure.change_tracker.addEdit([self.store_start, self.store_end, "Move"])
 
     def addOffset(self, pos: Sequence, dir: int, keep_aspect_ratio: bool = True):
         """ move the whole selection (e.g. for the use of the arrow keys) """
@@ -431,12 +528,14 @@ class GrabbableRectangleSelection(GrabFunctions):
         for target in self.targets:
             self.transform_target(transform, target)
 
-        for rect in self.targets_rects:
-            self.transform_target(transform, TargetWrapper(rect))
+        self.update_selection_rectangles(True)
+        #for rect in self.targets_rects:
+        #    self.transform_target(transform, TargetWrapper(rect))
 
     def move(self, pos: Sequence[float], dir: int, snaps: Sequence[SnapBase], keep_aspect_ratio: bool = False, ignore_snaps: bool = False):
         """ called from a grabber to move the selection. """
         self.addOffset(pos, dir, keep_aspect_ratio)
+        self.has_moved = True
 
         if not ignore_snaps:
             offx, offy = checkSnaps(snaps)
@@ -446,8 +545,6 @@ class GrabbableRectangleSelection(GrabFunctions):
 
         checkSnapsActive(snaps)
 
-        self.figure.canvas.draw()
-
     def apply_transform(self, transform: np.ndarray, point: Sequence[float]):
         """ apply the given transformation to a point"""
         point = np.array(point)
@@ -478,24 +575,31 @@ class GrabbableRectangleSelection(GrabFunctions):
         if event.key == 'left':
             self.start_move()
             self.addOffset((-1, 0), self.dir)
+            self.has_moved = True
             self.end_move()
+            self.figure.canvas.schedule_draw()
         if event.key == 'right':
             self.start_move()
             self.addOffset((+1, 0), self.dir)
+            self.has_moved = True
             self.end_move()
+            self.figure.canvas.schedule_draw()
         if event.key == 'down':
             self.start_move()
             self.addOffset((0, -1), self.dir)
+            self.has_moved = True
             self.end_move()
+            self.figure.canvas.schedule_draw()
         if event.key == 'up':
             self.start_move()
             self.addOffset((0, +1), self.dir)
+            self.has_moved = True
             self.end_move()
+            self.figure.canvas.schedule_draw()
         if event.key == "delete":
             for target in self.targets[::-1]:
                 self.figure.change_tracker.removeElement(target.target)
             self.figure.canvas.draw()
-        #print("event", event.key)
 
 
 class DragManager:
@@ -503,7 +607,7 @@ class DragManager:
     selected_element = None
     grab_element = None
 
-    def __init__(self, figure: Figure):
+    def __init__(self, figure: Figure, no_save):
         self.figure = figure
         self.figure.figure_dragger = self
 
@@ -534,9 +638,9 @@ class DragManager:
         for patch in self.figure.patches:
             self.make_dragable(patch)
 
-        self.selection = GrabbableRectangleSelection(figure)
+        self.selection = GrabbableRectangleSelection(figure, figure._pyl_scene)
         self.figure.selection = self.selection
-        self.change_tracker = ChangeTracker(figure)
+        self.change_tracker = ChangeTracker(figure, no_save)
         self.figure.change_tracker = self.change_tracker
 
     def activate(self):
@@ -634,7 +738,6 @@ class DragManager:
         # if there is a new element, select it
         self.on_select(element, event)
         self.selected_element = element
-        self.figure.canvas.draw()
 
     def on_deselect(self, event: MouseEvent):
         """ deselect currently selected artists"""
@@ -648,15 +751,31 @@ class DragManager:
         if element is not None:
             self.selection.add_target(element)
 
+    def undo(self):
+        print("back edit")
+        self.figure.change_tracker.backEdit()
+        self.selection.clear_targets()
+        self.selected_element = None
+        self.on_select(None, None)
+        self.figure.canvas.draw()
+
+    def redo(self):
+        print("forward edit")
+        self.figure.change_tracker.forwardEdit()
+        self.selection.clear_targets()
+        self.selected_element = None
+        self.on_select(None, None)
+        self.figure.canvas.draw()
+
     def key_press_event(self, event: KeyEvent):
         """ when a key is pressed """
         # space: print code to restore current configuration
         if event.key == 'ctrl+s':
             self.figure.change_tracker.save()
         if event.key == "ctrl+z":
-            self.figure.change_tracker.backEdit()
+            self.undo()
         if event.key == "ctrl+y":
-            self.figure.change_tracker.forwardEdit()
+            self.redo()
         if event.key == "escape":
             self.selection.clear_targets()
             self.selected_element = None
@@ -691,36 +810,97 @@ class GrabberGeneric(GrabFunctions):
         self.set_xy((self.ox+pos[0], self.oy+pos[1]))
 
 
-class GrabberGenericRound(Ellipse, GrabberGeneric):
+class GrabberGenericRound(GrabberGeneric):
     """ a rectangle with a round appearance """
     d = 10
+    shape = "round"
+
+    def __init__(self, parent: GrabbableRectangleSelection, x: float, y: float, dir: int, scene):
+        pen3 = QtGui.QPen(QtGui.QColor("black"), 2)
+        brush1 = QtGui.QBrush(QtGui.QColor("red"))
+
+        self.ellipse = MyEllipse(x, y, 10, 10, scene)
+        self.ellipse.view = scene.view
+        self.ellipse.grabber = self
+        self.ellipse.setPen(pen3)
+        self.ellipse.setBrush(brush1)
+        self.center = (x, y)
 
-    def __init__(self, parent: GrabbableRectangleSelection, x: float, y: float, dir: int):
         GrabberGeneric.__init__(self, parent, x, y, dir)
-        Ellipse.__init__(self, (0, 0), self.d, self.d, picker=True, figure=parent.figure, edgecolor="k", facecolor="r", zorder=1000, label="grabber")
-        self.figure.patches.append(self)
-        self.updatePos()
+
+    def set_xy(self, xy: (float, float)):
+        self.xy = xy
+        self.ellipse.setRect(xy[0]-5, xy[1]-5, 10, 10)
 
 
-class GrabberGenericRectangle(Rectangle, GrabberGeneric):
+class GrabberGenericRectangle(GrabberGeneric):
     """ a rectangle with a square appearance """
     d = 10
+    shape = "rect"
 
-    def __init__(self, parent: GrabbableRectangleSelection, x: float, y: float, dir: int):
+    def __init__(self, parent: GrabbableRectangleSelection, x: float, y: float, dir: int, scene):
         # somehow the original "self" rectangle does not show up in the current matplotlib version, therefore this doubling
-        self.rect = Rectangle((0, 0), self.d, self.d, figure=parent.figure, edgecolor="k", facecolor="r", zorder=1000, label="grabber")
-        self.rect._no_save = True
-        parent.figure.patches.append(self.rect)
+        #self.rect = Rectangle((0, 0), self.d, self.d, figure=parent.figure, edgecolor="k", facecolor="r", zorder=1000, label="grabber")
+        #self.rect._no_save = True
+        #parent.figure.patches.append(self.rect)
+
+        #Rectangle.__init__(self, (0, 0), self.d, self.d, picker=True, figure=parent.figure, edgecolor="k", facecolor="r", zorder=1000, label="grabber")
+
+        #self.figure.patches.append(self)
+
+        pen3 = QtGui.QPen(QtGui.QColor("black"), 2)
+        brush1 = QtGui.QBrush(QtGui.QColor("red"))
+
+        self.ellipse = MyRect(x-5, y-5, 10, 10, scene)
+        self.ellipse.view = scene.view
+        self.ellipse.grabber = self
+        self.ellipse.setPen(pen3)
+        self.ellipse.setBrush(brush1)
+
+        self.xy = (x, y)
+        #self.updatePos()
 
-        Rectangle.__init__(self, (0, 0), self.d, self.d, picker=True, figure=parent.figure, edgecolor="k", facecolor="r", zorder=1000, label="grabber")
         GrabberGeneric.__init__(self, parent, x, y, dir)
-        self.figure.patches.append(self)
-        self.updatePos()
 
     def get_xy(self):
+        return self.xy
         xy = Rectangle.get_xy(self)
         return xy[0] + self.d / 2, xy[1] + self.d / 2
 
     def set_xy(self, xy: (float, float)):
+        self.xy = xy
+
+        self.ellipse.setRect(xy[0] - 5, xy[1] - 5, 10, 10)
+        return
         Rectangle.set_xy(self, (xy[0] - self.d / 2, xy[1] - self.d / 2))
         self.rect.set_xy((xy[0] - self.d / 2, xy[1] - self.d / 2))
+
+class MyItem:
+    w = 10
+
+    def mousePressEvent(self, e):
+        super().mousePressEvent(e)
+        self.view.grabber_found = True
+        p = e.scenePos()
+        self.scene().grabber_pressed = self
+        self.grabber.button_press_event(MyEvent(p.x(), self.view.h - p.y()))
+
+    def mouseReleaseEvent(self, e):
+        super().mouseReleaseEvent(e)
+        self.scene().grabber_pressed = None
+        self.view.grabber_found = True
+        p = e.scenePos()
+        self.grabber.button_release_event(MyEvent(p.x(), self.view.h - p.y()))
+
+class MyRect(MyItem, QtWidgets.QGraphicsRectItem):
+    pass
+
+class MyEllipse(MyItem, QtWidgets.QGraphicsEllipseItem):
+    pass
+
+
+
+class MyEvent:
+    def __init__(self, x, y):
+        self.x = x
+        self.y = y
\ No newline at end of file
