@@ -25,7 +25,10 @@ from matplotlib.text import Text
 import numpy as np
 import traceback
 from .parse_svg import svgread
-from matplotlib.axes._subplots import Axes
+try:  # starting from mpl version 3.6.0
+    from matplotlib.axes import Axes
+except:
+    from matplotlib.axes._subplots import Axes
 from matplotlib.figure import Figure
 from .pyjack import replace_all_refs
 import os
@@ -355,8 +358,12 @@ def convertFromPyplot(old, new):
 
     str(new)  # important! (for some reason I don't know)
     for ax in old.axes:
-        old.delaxes(ax)
-        new._axstack.add(new._make_key(ax), ax)
+        #old.delaxes(ax)
+        ax.remove()
+        ax.figure = new
+        new.axes.append(ax)
+        new.add_axes(ax)
+        #new._axstack.add(new._make_key(ax), ax)
         new.bbox._parents.update(old.bbox._parents)
         new.dpi_scale_trans._parents.update(old.dpi_scale_trans._parents)
         replace_all_refs(old.bbox, new.bbox)
@@ -565,6 +572,8 @@ def axes_to_grid(axes=None, track_changes=False):
     if axes is None:
         fig = plt.gcf()
         axes = fig.axes
+    if len(axes) == 0:
+        return
 
     # get width and heights
     width = np.mean([ax.get_position().width for ax in axes])
@@ -586,7 +595,6 @@ def axes_to_grid(axes=None, track_changes=False):
                 new_indices[i] = np.argmin(d)
         axes_indices.append(new_indices)
     # sort the indices
-    pos = np.array(pos)
     for i in [0, 1]:
         sorted_indices = np.argsort(pos[i], axis=0)
         for indices in axes_indices:
@@ -649,3 +657,12 @@ def axes_to_grid(axes=None, track_changes=False):
         if track_changes is True:
             ax.figure.change_tracker.addChange(ax, ".spines['right'].set_visible(False)")
             ax.figure.change_tracker.addChange(ax, ".spines['top'].set_visible(False)")
+
+
+def main_figure(artist):
+    if artist is None:
+        return None
+    if artist.figure == artist:
+        return artist
+    else:
+        return main_figure(artist.figure)
\ No newline at end of file
