@@ -213,6 +213,8 @@ def apply_style(style: dict, patch: mpatches.Patch) -> dict:
                 pass
             elif key == "display":
                 pass
+            elif key == "text-anchor":
+                pass
             else:
                 print("ERROR: unknown style key", key, file=sys.stderr)
         except ValueError:
@@ -291,6 +293,12 @@ def patch_rect(node: minidom.Element, trans: mtransforms.Transform, style: dict,
     """ draw a svg rectangle node as a rectangle patch element into the figure (with the given transformation and style) """
     if node.getAttribute("d") != "":
         return patch_path(node, trans, style, ids)
+    if node.getAttribute("ry") != "" and node.getAttribute("ry") != 0:
+        return mpatches.FancyBboxPatch(xy=(float(node.getAttribute("x")), float(node.getAttribute("y"))),
+                                       width=float(node.getAttribute("width")),
+                                       height=float(node.getAttribute("height")),
+                                       boxstyle=mpatches.BoxStyle.Round(0, float(node.getAttribute("ry"))),
+                                       transform=trans)
     return mpatches.Rectangle(xy=(float(node.getAttribute("x")), float(node.getAttribute("y"))),
                               width=float(node.getAttribute("width")),
                               height=float(node.getAttribute("height")),
@@ -320,15 +328,32 @@ def plt_draw_text(node: minidom.Element, trans: mtransforms.Transform, style: di
     """ draw a svg text node as a text patch element into the figure (with the given transformation and style) """
     trans = parseTransformation(node.getAttribute("transform")) + trans + plt.gca().transData
     trans = mtransforms.Affine2D([[1, 0, 0], [0, -1, 0], [0, 0, 1]]) + trans
-    pos = np.array([svgUnitToMpl(node.getAttribute("x")), -svgUnitToMpl(node.getAttribute("y"))])
+    if node.getAttribute("x") != "":
+        pos = np.array([svgUnitToMpl(node.getAttribute("x")), -svgUnitToMpl(node.getAttribute("y"))])
+    else:
+        pos = np.array([0, 0])
 
     style = get_inline_style(node, get_css_style(node, ids["css"], style))
 
+    dx = node.getAttribute("dx") or "0"
+    dy = node.getAttribute("dy") or "0"
+
     text_content = ""
     patch_list = []
     for child in node.childNodes:
-        text_content += child.firstChild.nodeValue
-        if 1:
+        if not isinstance(child, minidom.Element):
+            partial_content = child.data
+            pos_child = pos.copy()
+
+            pos_child[0] += svgUnitToMpl(dx)
+            pos_child[1] -= svgUnitToMpl(dy)
+            style_child = style
+            part_id = ""
+        else:
+            part_id = node.getAttribute("id")
+            if child.firstChild is None:
+                continue
+            partial_content = child.firstChild.nodeValue
             style_child = get_inline_style(child, get_css_style(child, ids["css"], style))
             pos_child = pos.copy()
             if child.getAttribute("x") != "":
@@ -337,22 +362,19 @@ def plt_draw_text(node: minidom.Element, trans: mtransforms.Transform, style: di
                 pos_child[0] += svgUnitToMpl(child.getAttribute("dx"))
             if child.getAttribute("dy") != "":
                 pos_child[1] -= svgUnitToMpl(child.getAttribute("dy"))
+
+            text_content += partial_content
             path1 = TextPath(pos_child,
-                             child.firstChild.nodeValue,
+                             partial_content,
                              prop=font_properties_from_style(style_child))
             patch = mpatches.PathPatch(path1, transform=trans)
 
             apply_style(style_child, patch)
             if not no_draw and not styleNoDisplay(style_child):
                 plt.gca().add_patch(patch)
-            if child.getAttribute("id") != "":
-                ids[child.getAttribute("id")] = patch
+            if part_id != "":
+                ids[part_id] = patch
             patch_list.append(patch)
-        else:
-            text = plt.text(float(child.getAttribute("x")), float(child.getAttribute("y")),
-                     child.firstChild.nodeValue,
-                     transform=trans)
-            apply_style(style, text)
 
     if node.getAttribute("id") != "":
         ids[node.getAttribute("id")] = patch_list
@@ -522,6 +544,8 @@ def patch_path(node: minidom.Element, trans: mtransforms.Transform, style: dict,
 
     patch_list = []
 
+    if len(verts) == 0:
+        return patch_list
     path = mpath.Path(verts, codes)
     patch_list.append(mpatches.PathPatch(path, transform=trans))
 
@@ -550,7 +574,7 @@ def svgUnitToMpl(unit: str, default=None) -> float:
     import re
     if unit == "":
         return default
-    match = re.match(r"^([-.\d]*)(\w*)$", unit)
+    match = re.match(r"^([-.\d]*)(\w*).*$", unit)
     if match:
         value, unit = match.groups()
         value = float(value)
@@ -655,10 +679,13 @@ def parseGroup(node: minidom.Element, trans: mtransforms.Transform, style: dict,
             link = child.getAttribute("xlink:href")
             im = openImageFromLink(link)
             if no_draw is False:
-                im_patch = plt.imshow(im[::-1], extent=[svgUnitToMpl(child.getAttribute("x")), svgUnitToMpl(child.getAttribute("x")) + svgUnitToMpl(child.getAttribute("width")),
-                                                        svgUnitToMpl(child.getAttribute("y")), svgUnitToMpl(child.getAttribute("y")) + svgUnitToMpl(child.getAttribute("height")),
-                                                        ], zorder=1)
-                patch_list.append(im_patch)
+                if child.getAttribute("x") != "":
+                    im_patch = plt.imshow(im[::-1], extent=[svgUnitToMpl(child.getAttribute("x")), svgUnitToMpl(child.getAttribute("x")) + svgUnitToMpl(child.getAttribute("width")),
+                                                            svgUnitToMpl(child.getAttribute("y")), svgUnitToMpl(child.getAttribute("y")) + svgUnitToMpl(child.getAttribute("height")),
+                                                            ], zorder=1)
+                else:
+                    pass#im_patch = plt.imshow(im[::-1], zorder=1)
+                #patch_list.append(im_patch)
         elif child.tagName == "metadata":
             pass  # we do not have to draw metadata
         else:
